from synergy.exception import AuthorizationError


__author__ = "Lisa Zangrando"
__email__ = "lisa.zangrando[AT]pd.infn.it"
__copyright__ = """Copyright (c) 2015 INFN - INDIGO-DataCloud
All Rights Reserved

Licensed under the Apache License, Version 2.0;
you may not use this file except in compliance with the
License. You may obtain a copy of the License at:

       http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
       either express or implied.
       See the License for the specific language governing
       permissions and limitations under the License."""


class LocalHostAuthorization(object):

    def authorize(self, context):
        server_addr = context.get("SERVER_NAME")
        remote_addr = context.get("REMOTE_ADDR")

        if not server_addr or not remote_addr or server_addr != remote_addr:
            raise AuthorizationError("You are not authorized!")
